package main

import (
	"log"

	"git.sr.ht/~michalr/go-satel"
)

func isBasicEventElementOkay(basicEventElement satel.BasicEventElement, allowedTypes []satel.ChangeType, allowedIndexes []int) bool {
	for _, allowedType := range allowedTypes {
		if allowedType == basicEventElement.Type {
			return true
		}
	}
	for _, allowedIndex := range allowedIndexes {
		if allowedIndex == basicEventElement.Index {
			return true
		}
	}
	return false
}

func FilterByTypeOrIndex(ev <-chan satel.Event, allowedTypes []satel.ChangeType, allowedIndexes []int) <-chan satel.Event {
	returnChan := make(chan satel.Event)

	if (len(allowedTypes) == 0) && (len(allowedIndexes) == 0) {
		// no allowed types == all types are allowed
		go func() {
			for e := range ev {
				returnChan <- e
			}
			close(returnChan)
		}()
	} else {
		go func() {
			for e := range ev {
				retEv := satel.Event{BasicEvents: make([]satel.BasicEventElement, 0)}
				for _, basicEventElement := range e.BasicEvents {
					if isBasicEventElementOkay(basicEventElement, allowedTypes, allowedIndexes) {
						retEv.BasicEvents = append(retEv.BasicEvents, basicEventElement)
					}
				}
				if len(retEv.BasicEvents) != 0 {
					returnChan <- retEv
				}
			}
			close(returnChan)
		}()
	}

	return returnChan
}

func FilterByLastSeen(ev <-chan satel.Event, dataStore *DataStore, logger *log.Logger) <-chan satel.Event {
	returnChan := make(chan satel.Event)

	go func() {
		for e := range ev {
			retEv := satel.Event{BasicEvents: make([]satel.BasicEventElement, 0)}
			for _, basicEventElement := range e.BasicEvents {
				lastSeen := dataStore.GetSystemState()
				val, ok := lastSeen[EventKey{basicEventElement.Type, basicEventElement.Index}]
				if !ok || val.Value != basicEventElement.Value {
					retEv.BasicEvents = append(retEv.BasicEvents, basicEventElement)
					// TODO: flush to disk only after the loop finishes
					dataStore.SetSystemState(EventKey{basicEventElement.Type, basicEventElement.Index}, EventValue{basicEventElement.Value})
				}
			}
			if len(retEv.BasicEvents) != 0 {
				returnChan <- retEv
			}
		}
		logger.Print("Satel disconnected.")
		close(returnChan)
	}()

	return returnChan
}
