package main

import (
	"html/template"
	"log"
	"net/http"
	"os"
	"sync"

	"git.sr.ht/~michalr/go-satel"
)

const (
	NotificationPartitionIndex = 0
)

type Sender interface {
	Send(msg GenericMessage, tpl *template.Template) error
}

type Sleeper interface {
	Sleep(ch chan<- interface{})
}

type ChatId interface {
	GetTgIds() *[]int64
}

type TgChatId struct {
	tgChatId int64
}

func (self TgChatId) GetTgIds() *[]int64 {
	return &[]int64{self.tgChatId}
}

func Consume(events <-chan GenericMessage) {
	go func() {
		for range events {
		}
	}()
}

func SendToTg(events <-chan GenericMessage, s Sender, wg *sync.WaitGroup, logger *log.Logger, tpl *template.Template) <-chan GenericMessage {
	returnEvents := make(chan GenericMessage)

	go func() {
		wg.Add(1)
		defer wg.Done()
		for e := range events {
			returnEvents <- e
			err := s.Send(e, tpl)
			if err != nil {
				// TODO: handle it better
				panic(err)
			}
		}
		close(returnEvents)
	}()

	return returnEvents
}

func doHttpNotification(url string, logger *log.Logger, wg *sync.WaitGroup) {
	wg.Add(1)
	defer wg.Done()
	if len(url) == 0 {
		return
	}
	req, err := http.NewRequest(http.MethodPost, url, nil)

	res, err := http.DefaultClient.Do(req)
	if err != nil {
		logger.Print("Could not POST ", url, ": ", err)
		return
	}
	logger.Print("Notified via HTTP with result ", res.StatusCode)
}

func NotifyViaHTTP(events <-chan GenericMessage, wg *sync.WaitGroup, logger *log.Logger) <-chan GenericMessage {
	returnEvents := make(chan GenericMessage)

	armCallbackUrl := os.Getenv("NOTIFY_URL_ARM")
	disarmCallbackUrl := os.Getenv("NOTIFY_URL_DISARM")
	go func() {
		wg.Add(1)
		defer wg.Done()
		for e := range events {
		inner:
			for _, basicElement := range e.Messages {
				if (basicElement.Index == NotificationPartitionIndex) && (basicElement.Type == satel.ArmedPartition) {
					if basicElement.Value == ArmedPartition_Armed {
						go doHttpNotification(armCallbackUrl, logger, wg)
					} else {
						go doHttpNotification(disarmCallbackUrl, logger, wg)
					}
					break inner
				}
			}
		}
		close(returnEvents)
	}()

	return returnEvents
}
